# copyright (C) 1997-2000 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: mystatus.tcl,v 1.2.1.3 2000/09/22 19:54:27 jfontain Exp $}


package provide mystatus [lindex {$Revision: 1.2.1.3 $} 1]
if {[lsearch -exact $auto_path /usr/lib]<0} {                         ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
    lappend auto_path /usr/lib
}
if {[catch {package require Mytcl} message]} {
    error "$message\nMytcl package does not seem to be installed:\nget it at http://jfontain.free.fr/"
}
package require miscellaneous 1


namespace eval mystatus {

    array set data {
        updates 0
        0,label display 0,type ascii 0,message \
{per second values are calculated for the last
poll period using the absolute values from the
server, and only for variables that make sense
(otherwise a ? is displayed in the column)}
        0,0 absolute 1,0 {per second}
        pollTimes {10 5 20 30 60 120 300}
        switches {--host 1 --password 1 --port 1 --user 1}
    }
    set file [open [file join $::packageDirectory(mystatus) mystatus.htm]]
    set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
    close $file

    proc initialize {optionsName} {
        upvar $optionsName options
        variable host
        variable user
        variable connection
        variable data

        set arguments {}
        set host 127.0.0.1                                                                                             ;# by default
        catch {set host $options(--host)}
        lappend arguments $host
        set user $::env(USER)                                                                                          ;# by default
        catch {set user $options(--user)}
        lappend arguments $user
        set password {}
        catch {set password $options(--password)}
        catch {lappend arguments $password}
        lappend arguments {}                                                                              ;# no preselected database
        set port 3306                                                                                                  ;# by default
        catch {set port $options(--port)}
        catch {lappend arguments $port}
        set data(identifier) mystatus($host)
        set connection [eval sql connect $arguments]
        set query [sql query $connection {show variables like 'version'}]
        scan [lindex [sql fetchrow $connection $query] 1] %u.%u.%u major minor subMinor
        sql endquery $connection $query
        if {$major<3} {
            error {cannot monitor a server below version 3}
        }
        if {$minor<22} {
            error {cannot monitor a server below version 3.22}
        } elseif {$subMinor<24} {
            error {cannot monitor a 3.23 server below version 3.23.24}
        }
        if {$minor>22} {
            array set data {
                1,label {aborted clients} 1,type real 1,message\
                "number of connections that have been\naborted because the client has died\nwithout closing the connection properly"
                2,label {aborted connects} 2,type real 2,message "number of tries to connect\nto the server that have failed"
                3,label <kilobytes 3,type real 3,message {number of kilobytes received}
                4,label kilobytes> 4,type real 4,message {number of kilobytes sent}
                5,label connections 5,type real 5,message {number of connection attempts to the server}
                6,label {disk temporaries} 6,type real 6,message\
                    "number of implicit temporary tables on disk that\nhave been created while executing statements"
                7,label {memory temporaries} 7,type real 7,message\
                    "number of implicit temporary tables in memory that\nhave been created while executing statements"
                8,label {delayed inserts} 8,type real 8,message {number of delayed insert handler threads in use}
                9,label {delayed writes} 9,type real 9,message {number of rows written with INSERT DELAYED}
                10,label {delayed errors} 10,type real 10,message\
                    "number of rows written with\nINSERT DELAYED for which some error\noccurred (probably duplicate key)"
                11,label flush 11,type real 11,message {number of executed FLUSH commands}
                12,label delete 12,type real 12,message {number of times a row was deleted from a table}
                13,label {read first} 13,type real 13,message \
{number of times the first entry was read from an index.
If this is high, it suggests that the server is doing
a lot of full index scans, for example,
SELECT col1 FROM foo, assuming that col1 is indexed}
                14,label {read key} 14,type real 14,message \
{number of requests to read a row based on a key.
If this is high, it is a good indication that
your queries and tables are properly indexed}
                15,label {read order} 15,type real 15,message \
{number of requests to read next row in key order.
This will be incremented if you are querying an
index column with a range contraint. This also
will be incremented if you are doing an index scan}
                16,label {read previous} 16,type real 16,message\
                    {number of requests to read previous row in key order (undocumented)}
                17,label {read fixed} 17,type real 17,message \
{number of requests to read a row based on a fixed position.
This will be high if you are doing a lot of queries that
require sorting of the result}
                18,label {read next} 18,type real 18,message \
{number of requests to read the next row in the datafile.
This will be high if you are doing a lot of table scans.
Generally this suggests that you tables are not properly
indexed or that you queries are not written to take
advantaged of the indices you have}
                19,label updates 19,type real 19,message {number of requests to update a row in a table}
                20,label inserts 20,type real 20,message {number of requests to insert a row in a table}
                21,label blocks 21,type integer 21,message {number of used blocks in the key cache} 1,21 ?
                22,label <cache 22,type real 22,message {number of requests to read a key block from cache}
                23,label <disk 23,type real 23,message {number of physical reads of a key block from disk}
                24,label cache> 24,type real 24,message {number of requests to write a key block to cache}
                25,label disk> 25,type real 25,message {number of physical writes of a key block to disk}
                26,label connections 26,type integer
                    26,message "maximum number of connections\nthat have been in use simultaneously" 1,26 ?
                27,label {unflushed keys} 27,type integer
                    27,message "keys blocks in the key cache that have\nchanged but have not yet been flushed to disk" 1,27 ?
                28,label {unflushed rows} 28,type integer 28,message {number of rows waiting to be written in INSERT DELAY queues}
                    1,28 ?
                29,label {open tables} 29,type integer 29,message {number of tables that are open} 1,29 ?
                30,label files 30,type integer 30,message {number of files that are open} 1,30 ?
                31,label streams 31,type integer 31,message "number of streams that are open\n(used mainly for logging)" 1,31 ?
                32,label {opened tables} 32,type integer 32,message {number of tables that have been opened} 1,32 ?
                33,label queries 33,type real 33,message {number of queries sent to the server}
                34,label {slow threads} 34,type real 34,message\
                    "number of threads that have taken more\nthan slow_launch_time to connect"
                35,label {slow queries} 35,type real 35,message "number of queries that have taken\nmore than long_query_time"
                36,label {slave running} 36,type ascii 36,message {(undocumented)} 1,36 {}
                37,label {cached threads} 37,type integer 37,message {number of threads in the thread cache} 1,37 ?
                38,label {connected threads} 38,type integer 38,message {number of currently open connections} 1,38 ?
                39,label {running threads} 39,type integer 39,message {number of threads that are not sleeping} 1,39 ?
                40,label uptime 40,type dictionary
                    40,message "time the server has been up\nin d(ays), h(ours), m(inutes) and s(econds)" 1,40 {}
            }
            set data(views) {
                {
                    indices
                    {
                        0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29
                        30 31 32 33 34 35 36 37 38 39 40
                    }
                    swap 1
                }
            }
        } else {
            array set data {
                1,label {aborted clients} 1,type real 1,message\
                "number of connections that have been\naborted because the client has died\nwithout closing the connection properly"
                2,label {aborted connects} 2,type real 2,message "number of tries to connect\nto the server that have failed"
                3,label temporary 3,type real 3,message\
                    "number of implicit temporary tables that have\nbeen created while executing statements"
                4,label {delayed inserts} 4,type real 4,message {number of delayed insert handler threads in use}
                5,label {delayed writes} 5,type real 5,message {number of rows written with INSERT DELAYED}
                6,label {delayed errors} 6,type real 6,message\
                    "number of rows written with\nINSERT DELAYED for which some error\noccurred (probably duplicate key)"
                7,label flush 7,type real 7,message {number of executed FLUSH commands}
                8,label delete 8,type real 8,message {number of requests to delete a row from a table}
                9,label {read first} 9,type real 9,message {number of requests to read the first row in a table}
                10,label {read key} 10,type real 10,message {number of requests to read a row based on a key}
                11,label {read next} 11,type real 11,message {number of requests to read next row in key order}
                12,label {read fixed} 12,type real 12,message {number of requests to read a row based on a fixed position}
                13,label updates 13,type real 13,message {number of requests to update a row in a table}
                14,label inserts 14,type real 14,message {number of requests to insert a row in a table}
                15,label blocks 15,type integer 15,message {number of used blocks in the key cache} 1,15 ?
                16,label <cache 16,type real 16,message {number of requests to read a key block from cache}
                17,label <disk 17,type real 17,message {number of physical reads of a key block from disk}
                18,label cache> 18,type real 18,message {number of requests to write a key block to cache}
                19,label disk> 19,type real 19,message {number of physical writes of a key block to disk}
                20,label connections 20,type integer
                    20,message "maximum number of connections\nthat have been in use simultaneously" 1,20 ?
                21,label {unflushed keys} 21,type integer
                    21,message "keys blocks in the key cache that have\nchanged but have not yet been flushed to disk" 1,21 ?
                22,label {unflushed rows} 22,type integer
                    22,message {number of rows waiting to be written in INSERT DELAY queues} 1,22 ?
                23,label {open tables} 23,type integer 23,message {number of tables that are open} 1,23 ?
                24,label files 24,type integer 24,message {number of files that are open} 1,24 ?
                25,label streams 25,type integer 25,message "number of streams that are open\n(used mainly for logging)" 1,25 ?
                26,label {opened tables} 26,type integer 26,message {number of tables that have been opened} 1,26 ?
                27,label queries 27,type real 27,message {number of queries sent to the server}
                28,label running 28,type integer 28,message {number of threads that are not sleeping} 1,28 ?
                29,label slow 29,type real 29,message "number of queries that have taken\nmore than long_query_time"
                30,label uptime 30,type dictionary
                    30,message "time the server has been up\nin d(ays), h(ours), m(inutes) and s(econds)" 1,30 {}
            }
            set data(views) {
                {indices {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30} swap 1}
            }
        }
    }

    proc update {} {
        variable connection
        variable last
        variable data

        if {[catch {set query [sql query $connection {show status}]} message]} {                          ;# problem reaching server
            flashMessage "mystatus error: $message"
            catch {unset last}
            foreach name [array names data *,label] {
                scan $name %u column
                if {$column==0} continue                                                                           ;# headers column
                set data(0,$column) ?
                if {[string compare $data($column,type) real]==0} {                                ;# required for per second values
                    set data(1,$column) ?
                }
            }
        } else {
            set clock [expr {[clock clicks]/1e6}]                                                          ;# store clock in seconds
            catch {set period [expr {$clock-$last(clock)}]}
            set last(clock) $clock
            set column 1
            while {[llength [set list [sql fetchrow $connection $query]]]>0} {
                foreach {variable value} $list {}
                if {[string compare $variable Uptime]==0} {
                    set data(0,$column) [formattedTime $value]
                    set data(1,$column) {}
                } else {
                    set data(0,$column) $value
                    if {[string compare $data($column,type) real]==0} {                            ;# required for per second values
                        if {[info exists period]} {
                            if {[string match *Bytes* $variable]} {                 ;# anything in bytes is transformed to kilobytes
                                set data(1,$column) [format %.2f [expr {($value-$last($column))/1000.0/$period}]]
                            } else {
                                set data(1,$column) [format %.2f [expr {($value-$last($column))/$period}]]
                            }
                        } else {
                            set data(1,$column) ?
                        }
                        set last($column) $value
                    }
                }
                incr column
            }
            sql endquery $connection $query
        }
        incr data(updates)
    }

    proc terminate {} {
        variable connection
        catch {sql disconnect $connection}
    }

}
